#include <ros/ros.h>
#include "imu_data.h"
#include <Eigen/Eigen>
#include <Eigen/Geometry>
#include <Eigen/Core>
#include <sensor_msgs/Imu.h>

using namespace Eigen;
using namespace std;

Eigen::Quaterniond euler2Quater(float roll, float yaw, float pitch)
{
    Eigen::AngleAxisd rollAngle(roll, Eigen::Vector3d::UnitZ());
    Eigen::AngleAxisd yawAngle(yaw, Eigen::Vector3d::UnitY());
    Eigen::AngleAxisd pitchAngle(pitch, Eigen::Vector3d::UnitX());

    Eigen::Quaterniond q = rollAngle * yawAngle * pitchAngle;
    cout << "Euler2Quaternion result is:" <<endl;
    cout << "x = " << q.x() <<endl;
    cout << "y = " << q.y() <<endl;
    cout << "z = " << q.z() <<endl;
    cout << "w = " << q.w() <<endl<<endl;

    return q;
}

int main(int argc, char **argv)
{
    ros::init(argc, argv, "imu_data_pub_node");
    ros::NodeHandle handle;
    myIMU my_imu;
    my_imu.initIICDevice();

    ros::Publisher imu_pub = handle.advertise<sensor_msgs::Imu>("imu_data", 2);
    ros::Rate loop_rate(10);
    sensor_msgs::Imu imu_msg;
    float yaw_tmp;
    float pitch_tmp;
    float roll_tmp;

    while(ros::ok())
    {
        imu_msg.header.stamp = ros::Time::now();
        imu_msg.header.frame_id = "base_link";

        roll_tmp  = my_imu.getRollData();
        pitch_tmp = my_imu.getPitchData();
        yaw_tmp   = my_imu.getYawData();

        Eigen::Quaterniond q = euler2Quater(roll_tmp, pitch_tmp, yaw_tmp);
        imu_msg.orientation.x = q.x();
        imu_msg.orientation.y = q.y();
        imu_msg.orientation.z = q.z();
        imu_msg.orientation.w = q.w();
        imu_msg.orientation_covariance = {0.0025, 0, 0,
                                          0, 0.0025, 0,
                                          0, 0, 0.0025};

        imu_msg.angular_velocity.x = my_imu.getGyoXData();
        imu_msg.angular_velocity.y = my_imu.getGyoYData();
        imu_msg.angular_velocity.z = my_imu.getGyoZData();
        imu_msg.angular_velocity_covariance = {0.02, 0, 0,
                                               0, 0.02, 0,
                                               0, 0, 0.02};

        imu_msg.linear_acceleration.x = my_imu.getAccXData();
        imu_msg.linear_acceleration.y = my_imu.getAccYData();
        imu_msg.linear_acceleration.z = my_imu.getAccZData();
        imu_msg.linear_acceleration_covariance = {0.04, 0, 0,
                                                   0, 0.04, 0,
                                                   0, 0, 0.04};

        imu_pub.publish(imu_msg);

        ros::spinOnce();
        loop_rate.sleep();
    }

    return 0;
}

